% This function approximates a density using a Gaussian kernel and
% evaluate it at z0
% See:
%
% Chan, J.C.C, Fry-McKibbin, R. A. and Hsiao, C. Y.-L. (2018). A Regime 
% Switching Skew-normal Model of Contagion, Studies in Nonlinear Dynamics
% and Econometrics, forthcoming

function logden = gaucopulanew060712(z0,z)
[M, n] = size(z);
c = 25*sqrt(var(z)/M);
p = zeros(M,n); P = zeros(M,n);
for i=1:M
    err = (repmat(z(i,:),M,1)-z)./repmat(c,M,1);
    p(i,:) = mean(normpdf(err))./c;
    P(i,:) = mean(normcdf(err));
end
w = norminv(P);
Sig = w'*w/M + eye(n)/10^6; %% ensure Sig is positive definite
err0 = (repmat(z0,M,1)-z)./repmat(c,M,1);
P0 = mean(normcdf(err0));
P0 = min(P0,1-10^(-15));
w0 = norminv(P0);
tempk = -.5*log(2*pi)-.5*err0.^2;
maxk = max(tempk);
logdf = maxk + log(mean(exp(tempk-repmat(maxk,M,1)))) - log(c) - log(normpdf(w0));
logden = -n/2*log(2*pi) - sum(log(diag(chol(Sig)))) -.5*(w0*(Sig\w0')) + sum(logdf);
end