% This function returns initial Z with N*m random variables
% See:
%
% Chan, J.C.C, Fry-McKibbin, R. A. and Hsiao, C. Y.-L. (2018). A Regime 
% Switching Skew-normal Model of Contagion, Studies in Nonlinear Dynamics
% and Econometrics, forthcoming

%% input:
% S: regimes that takes the values of 1 (non-crisis state) and 2 (crisis state)
% shortY: sample takes N*m matrix
% Del1: skewness and co-skewness m*m matrix in the non-crisis state
% mu1: mean parameter m*1 vector in the non-crisis state
% Del2: skewness and co-skewness m*m matrix in the non-crisis state
% mu2: mean parameter m*1 vector in the non-crisis state
% invSig1: inverse variance covariance m*m matrix in the non-crisis state
% invSig2: inverse variance covariance m*m matrix in the crisis state

function Z = initialZ(S,shortY,Del1,mu1,Del2,mu2,invSig1,invSig2)
[N,m]= size(shortY);
id1 = find(S==1);   id2 = find(S==2);
n1 = length(id1);   n2 = N - n1;
shortY1 = shortY(id1,:);         shortY2 = shortY(id2,:);
c = -sqrt(2/pi);
Z = reshape(tnormrnd(c,1,c,inf,N*m),N,m);
for i=1:2000
    DelinvSig1 = Del1'*invSig1;
    Dz1 = (speye(m) + DelinvSig1*Del1)\speye(m); % common covariance for all Z_i's 
    dz1 = c + DelinvSig1*(shortY1' - repmat(mu1,1,n1));
    Zmu1 = (Dz1*dz1)';
    for j = 1:m        
        sigj = Dz1(j,[1:j-1 j+1:end]);
        invSigjj = Dz1([1:j-1 j+1:end], [1:j-1 j+1:end])\speye(m-1);
        Zerr = Z(id1,:) - Zmu1;
        Zconmu = Zerr(:,[1:j-1 j+1:end]);            
        muj = Zmu1(:,j) + (sigj*invSigjj*Zconmu')'; % compute the condistional mean
        vj = Dz1(j,j) - sigj*invSigjj*sigj';        
            % check if there're any inf values
        Zj = Z(id1,j);
        tempZ = tnormrnd(muj,vj*ones(n1,1),c,inf,n1);
        id = find(isinf(tempZ)==1);
        tempZ(id) = Zj(id);
        Z(id1,j) = tempZ; 
    end    
    DelinvSig2 = Del2'*invSig2;
    Dz2 = (speye(m) + DelinvSig2*Del2)\speye(m);
    dz2 = c + DelinvSig2*(shortY2' - repmat(mu2,1,n2));
    Zmu2 = (Dz2*dz2)';    
    for j = 1:m        
        sigj = Dz2(j,[1:j-1 j+1:end]);
        invSigjj = Dz2([1:j-1 j+1:end], [1:j-1 j+1:end])\speye(m-1);
        Zerr = Z(id2,:) - Zmu2;
        Zconmu = Zerr(:,[1:j-1 j+1:end]);            
        muj = Zmu2(:,j) + (sigj*invSigjj*Zconmu')'; % compute the condistional mean
        vj = Dz2(j,j) - sigj*invSigjj*sigj';        
            % check if there're any inf values
        Zj = Z(id2,j);
        tempZ = tnormrnd(muj,vj*ones(n2,1),c,inf,n2);
        id = find(isinf(tempZ)==1);
        tempZ(id) = Zj(id);
        Z(id2,j) = tempZ;
    end     
end