% This function takes Z and constructs a large sparse matrix B so that 
% Delta Z = B vech(Delta)
% See:
%
% Chan, J.C.C, Fry-McKibbin, R. A. and Hsiao, C. Y.-L. (2018). A Regime 
% Switching Skew-normal Model of Contagion, Studies in Nonlinear Dynamics
% and Econometrics, forthcoming

function B = makebigX(Z)
[N, m] = size(Z);
A = reshape(1:N*m,m,N)';
B =[];
for i=1:m
    longZ = reshape(Z(:,i:end)',1,N*(m-i+1));
    longZi = kron(Z(:,i),ones(m-i,1));
    id1i = kron(i:m:N*m,ones(1,m-i+1));
    id1j = repmat(1:m-i+1,1,N);
    id2i = reshape(A(:,i+1:end)',1,N*(m-i));
    id2j = repmat(2:m-i+1,1,N);
    temp = sparse([id1i id2i],[id1j id2j],[longZ longZi'],N*m,m-i+1);
    B = [B temp];
end